/******************************************************************************
	Lava Effect by Sergey A. Makovkin (sergeymak@pisem.net)
	hg changed Time and using texCoord instead of world coord 

  Adapted for PureBasic by Kelebrindae
******************************************************************************/

// Matrices
float4x4 WorldInverseTranspose : WorldInverseTranspose;
float4x4 WorldViewProjection : WorldViewProjection;
float4x4 World : World;
float4x4 ViewInverse : ViewInverse;

float Time;

/* data passed from vertex shader to pixel shader */
struct vertexOutput {
    float4 HPosition	: POSITION;
    float4 TexCoord	: TEXCOORD0;    
    float3 WorldNormal	: TEXCOORD2;
    float3 WorldPos	: TEXCOORD3;
    float3 WorldEyePos	: TEXCOORD4;
};


/*********** vertex shader ******/
void mainVS(in float3 _Position	: POSITION,
            in float4 _UV		: TEXCOORD0,
            in float4 _Normal	: NORMAL,
            out vertexOutput OUT,
            uniform float4x4 WorldViewProjection,
            uniform float4x4 WorldInverseTranspose,
            uniform float4x4 World,
            uniform float4x4 ViewInverse
            ) 
{
    OUT.WorldNormal = mul(WorldInverseTranspose,_Normal ).xyz;
    float3 WorldSpacePos = mul(World,float4(_Position, 1) ).xyz;
    OUT.WorldPos = WorldSpacePos;
    OUT.TexCoord = _UV;
    OUT.WorldEyePos = ViewInverse[3].xyz;
    OUT.HPosition = mul(WorldViewProjection,float4(_Position, 1) );
}

/*********** pixel shader ******/

void mainPS(in vertexOutput IN,
            out float4 _colour : COLOR0,
            uniform float Time,
            uniform float4 LavaColor1,
            uniform float4 LavaColor2,
            uniform float LavaFactor,
            uniform sampler2D chunkSampler : TEXUNIT0
            ) 
{

	// Sample 4 octaves of noise
	
	float rnd = 0.0f;
	float f = 1;
	float3 Coord = IN.TexCoord;
	Coord.y -= .1f*Time; 
	Coord.z = Time;	   // hg 
	for (int i = 0; i < 4; i++)
	{
		half fnoise = tex2D(chunkSampler, Coord * .2f * f).r;
		fnoise -=.5f;
		fnoise *= 4.0f;
		rnd += ( fnoise) / f;
		f *= 4.17;	
	}
	    
	float3 coord = IN.TexCoord;		 // was  IN.WorldPos 
	coord.x += rnd*LavaFactor;
	coord.y += rnd*LavaFactor;
	
	float4 tex = tex2D(chunkSampler, coord);
	
	// Add the terms
  _colour = tex * LavaColor1 * (rnd * 0.1) * 10 + LavaColor2; 
  
	// funny noise :
	//_colour = rnd;

}

void mainGlowPS(in vertexOutput IN,
                out float4 _colour : COLOR0,
                      uniform float Time,
                      uniform float4 LavaColor1,
                      uniform float LavaFactor,
                      uniform sampler2D chunkSampler : TEXUNIT0
                      )
{
    
	// Sample 4 octaves of noise
	
	float rnd = 0.0f;
	float f = 1;
	float3 Coord = IN.TexCoord;
	Coord.y -= .1f*Time; 
	for (int i = 0; i < 4; i++)
	{
		half fnoise = tex2D(chunkSampler, Coord * .2f * f).r;

		fnoise -=.5f;
		fnoise *= 4.0f;
		rnd += ( fnoise) / f;
		f *= 4.17;	
	}
	    
	float3 coord = IN.WorldPos;
	coord.x += rnd*LavaFactor;
	coord.y += rnd*LavaFactor;
	float4 tex = tex2D(chunkSampler, coord);
				
	// Add the terms
	_colour = tex * LavaColor1 * (rnd + 0.1) * 0.5;    

}

